c
c ===========================================================================
c    Simple solution of cracck boundary value problem
c    following Madariaga, Olsen and Archuleta, BSSA 1998
c ==========================================================================
c
c Originally programmed by  
c Kim Bak Olsen ICS/UC Santa Barbara, June 1996.
c Modified by R. MAdariaga and K.B. Olsen S.B. 1997
c 
c
c  DISTRIB 2.0  Madariaga June 2007 (DNS SRC)
c  fd3d_xy_thin is a new version using thin B.C on the fault
c  Done at S.B. in August 2006
c          Paris May-June 2007
c
      parameter (ntt=10000,nl=2)

      include 'parstat'

      dimension vpe(nl),vse(nl),den(nl),src(ntt),src1(ntt),
     +     nbhx(100),nbhy(100),nbhz(100)
      common/strini/ strinix(nx,ny),striniy(nx,ny)
      integer broken(nx,ny),incrack(nx,ny)
      real ruptime(nx,ny)

      nxt=nx
      nyt=ny
      nzt=nz
c
cc   open files
c
      call open3d(1)
c
ccc   read parameters
c
      
      call rddt3d(nt,dh,dt,
     +     nbgx,nedx,nskpx,nbgy,nedy,nskpy,
     +     ntiskp,ndamp,nsv,ifre,nxsc,nysc,nzsc,nholes,nbhx,nbhy,
     +     nbhz,idebug,dmax,vmax)

c
c     print them out
c
      print *
      print *, '  ===================== '
      print *, '  FD3D du 18 Juin 2007  '
      print *, '  R. Madariaga '
      print *, '  ===================== '
      print *
      print *, 'THIN Boundary Condition'
      print *,' Classical triangular Slip weakening law'
      print *
      print *,' compilation size nx, ny, nz = ',nx,ny,nz
      print *,' grid size :  nxt,nyt,nzt = ', nxt,nyt,nzt
      print *,' center of fault: nxrsc,nysc,nzsc = ', nxsc,nysc,nzsc
      print *,' debug = ',idebug
      print *,' dmax = ',dmax
      print *,' vmax = ',vmax
c
c   initialize the different arrays 
c
      do i=1,nx
         do j=1,ny
            broken(i,j)=0
            incrack(i,j)=0.
            ruptime(i,j)=0.
            strinix(i,j)=strout
            striniy(i,j)=0.
            peak_xz(i,j)=peak
            peak_yz(i,j)=peak
         enddo
      enddo
c
c    Read the stress field from external files
c
      open(3,file="stressx.ini")
      read(3,*) nnx,nny
      if(nnx.ne.nx.or.nny.ne.ny) 
     1     write(6,*) 'initial stress file is wrong ',
     $     'number of points less than ',nny,' or ',nny
      read(3,*) ((strinix(i,j),i=1,nnx),j=1,nny)
      close(3)
c
c   read rupture resistance along x-axis
c
      open(3,file="peakxz.ini")
      read(3,*) nnx,nny
      if(nnx.ne.nx.or.nny.ne.ny) then 
         write(6,*) 'peak stress file is wrong ',
     $        'it has ',nny,' by ',nny,' points '
         write(6,*)' the program is dimensionned ',nx,' by ',ny
      endif
      read(3,*) ((peak_xz(i,j),i=1,nnx),j=1,nny)
      close(3)
      
c     
ccc   read model
c     
      call rdmd3d(vpe,vse,den,nxt,nyt,nzt,ifre)
c
      print *,' velocities '
      write(*,*)'P wave ',vpe(1),vpe(2)
      write(*,*)'S wave ',vse(1),vse(2)
      write(*,*)'density',den(1),den(2)
      cfl=vpe(2)*dt/dh
      print *,' dt, dh, cfl = ',dt,dh,cfl
c     
ccc   check write out of parameters
c     
      call wrpm3d(nxt,nyt,nzt,nt,dh,dt,nsou,
     +     ndamp,ntiskp,nbgx,nedx,nskpx,nbgy,nedy,
     +     nskpy,vpe,vse,den,ifre,nxsc,nysc,nzsc,
     +     peak,ruptur,rupmax, strout, strasp,itype,
     +     dmax,vmax, xmin,xmax,ymin,ymax )
c     
c     -------------------------------------------------------
c     Loop over time 
c     -------------------------------------------------------
c
      do 10 ntst = 1,nt
         time =ntst*dt
         print *,' ntime step, time = ', ntst,time  
c
c  -------------------------------------------------------------
c    Apply 4th-order differencing to interior particle velocities
c             from  2:nxt;2:nyt;2:nzt
c  -------------------------------------------------------------
c

         call dvel(nxt,nyt,nzt,dh,dt)
c
ccc   compute absorbing boundary conditions in velocities
c
         call abc(nxt,nyt,nzt,dt,dh)
c
ccc   compute velocities of 2nd order accuracy at boundary
c
         call bnd2d(nxt,nyt,nzt,dt,dh)
c      
ccc   exponential damping along boundaries
c
c         call dahs3d(nxt,nyt,nzt,ndamp)
c
c  -------------------------------------------------------------
ccc   4th-order differencing of pressure
c  -------------------------------------------------------------
c
         call dstres(nxt,nyt,nzt,dh,dt)
c
cc   compute stress tensor of 2nd order accuracy at boundary
c
         call strbnd(nxt,nyt,nzt,dh,dt)
c
c   ======================================================================
c           CRACK BOUNDARY CONDITIONS  (applied right after stress update)
c   ======================================================================
c
c   The boundary conditions are applied now in ABSOLUTE STRESS, not in
c     relative stress, otherwise we get all sort of problems
c
 
        do i=1,nxt
           do j=1,nyt
c
              tabsx = xz(i,j,nzsc)+strinix(i,j)
              tabsy = yz(i,j,nzsc)+striniy(i,j)
              tabs = sqrt(tabsx**2+tabsy**2)
c              phi = atan2(tabsy,tabsx)
              if(tabsx.gt.peak_xz(i,j)) then
                 broken(i,j)=1
                 incrack(i,j)=1
                 ruptime(i,j)=time
c                 if(idebug.eq.1)then
c                    print *,i,j, ruptime(i,j)
c                 endif
              endif
           enddo
        enddo
c
c  Apply Boundary Conditions
c
        k=nzsc
        do i=1,nxt
           do j=1,nyt
c
c   Do boundary conditions only for points that are considered broken,
c   i.e. those that are slipping
c
              if (incrack(i,j).eq.1) then
                 if (broken(i,j).eq.1) then
                    if(u1(i,j,k+1).lt.0.) then
                       broken(i,j)=0.
c                       print *, "healed ",i,j,'vel = ',u1(i,j,nzsc+1),
c     +                      u1(i,j,nzsc),u1(i,j,nzsc-1)
                       u1(i,j,k)=0.
                       v1(i,j,k)=0.
                    else
c
c   if the point is slipping compute slip, sliprate
c     
                       sliprate = u1(i,j,nzsc+1)
                       slip = u2(i,j,nzsc+1)
c
c   Apply friction law to absolute slip, absolute sliprate and absolute
c       traction
c
c   Classical slip weakening law
c

                       if(dmax.eq.0.)then
                          dd=0.
                       else
                          if(slip.le.dmax)then
                             dd=1-slip/dmax
                          else
                             dd=0.
                          endif
                       endif
                       if (vmax.eq.0.) then
                          dv=0.
                       else
                          dv=vmax/(vmax+sliprate)
                       endif
                       if (dv.gt.dd) dd=dv
c
c    This is the absolute value of friction
c
                       friction=peak_xz(i,j)*dd
c THIN B.C. here:
                       xz(i,j,k)=friction -strinix(i,j) 
c THICB B.C. =>        xz(i,j,k-1)=friction-strinix(i,j)
                       if(idebug.eq.1)then  
                          print *,i,j,' (u1,v1), theta, (xz,yz) =',
     +                         u1(i,j,nzsc+1), v1(i,j,nzsc+1), theta,
     $                         xz(i,j,nzsc+1),yz(i,j,nzsc+1)
                        endif
                     endif
                 endif
              endif
           enddo
        enddo
c
c      computing displacements everywhere (Do we need this?)
c
         do k=1,nzt
            do j=1,nyt
               do i=1,nxt
                  u2(i,j,k)=u2(i,j,k)+u1(i,j,k)*dt
                  v2(i,j,k)=v2(i,j,k)+v1(i,j,k)*dt
                  w2(i,j,k)=w2(i,j,k)+w1(i,j,k)*dt
               enddo
            enddo
         enddo
c
ccc   write out seismograms
c
         call wrss3d(ntst,ntiskp,nxt,nyt,nzt,dt,dh,nholes,nbhx,nbhy,
     +        nbhz,nbgx,nedx,nskpx,nbgy,nedy,nskpy,ifre,ndamp,
     $        nxsc,nysc,nzsc)

c
 10   continue
      open(88,file="result/ruptime.res")
      do i=1,nxt
         do j=1,nyt
            write(88,*)ruptime(i,j)
         enddo
         write(88,'()')
      enddo
      close(88)
c
c -------------------------------------------------------------
c   END main computational loop
c -------------------------------------------------------------
c
cc   close files
c
      call open3d(0)
c
      end
c
c   ====================================================================
c              READ MODEL 3d
c   ====================================================================
c
      subroutine rdmd3d(vpe,vse,den,nxt,nyt,nzt,ifre)

ccc   routine to read the model 
c     nxt     # of node points in x-direction (integer)(sent)
c     nyt     # of node points in y-direction (integer)(sent)
c     nzt     # of node points in z-direction (integer)(sent)
c     vpe(1)  lowest p-velocity               (real)   (returned)
c     vpe(2)  highest p-velocity              (real)   (returned)
c     vse(1)  lowest s-velocity               (real)   (returned)
c     vse(2)  highest s-velocity              (real)   (returned)
c     den(1)  lowest density                  (real)   (returned)
c     den(2)  highest density                 (real)   (returned)
c     ifre    flag; =0 flat fs, <>0  topo     (integer)(returned)

      include 'parstat'
      dimension vpe(*),vse(*),den(*)
      vpe(2)=0.
      vpe(1)=1.0E10 
      vse(2)=0.
      vse(1)=1.0E10 
      den(2)=0.
      den(1)=1.0E10 
c
c LAYERED MEDIUM 
c

      open(10,file='model.dat')
      write(21,*)' Opened model.dat for reading velocity model'  
      write(*,*)' Reads velocity density model from model.dat'  
      do  k=nz,1,-1
         read(10,*) vp,vs,dd
         if (vp.gt.vpe(2)) vpe(2)=vp
         if (vp.lt.vpe(1)) vpe(1)=vp
         if (vs.gt.vse(2)) vse(2)=vs
         if (vs.lt.vse(1)) vse(1)=vs
         if (dd.gt.den(2)) den(2)=dd
         if (dd.lt.den(1)) den(1)=dd
c
c distributes these values on every layer of the model
c
         do j=1,ny
            do i=1,nx      
               lam1(i,j,k)=dd*(vp**2-2.*vs**2)
               mu1(i,j,k)=dd*vs**2
               d1(i,j,k)=dd
            enddo
         enddo
      enddo
      close(10)
      
      if (ifre.ne.0) then
         
         do  181 j=1,ny
            do 181 i=1,nx
               do 180 k=nz,1,-1
                  if (lam1(i,j,k).gt.0.) then
                     jj1(i,j)=k
                     goto 181
                  endif
 180           continue
 181        continue
         endif
         
         goto 20
   15 stop 'end of model file'
   20 continue
      return  
      end 
c
c   ====================================================================
c              READ DATA 3d
c   ====================================================================
c
      subroutine rddt3d(nt,dh,dt,
     +     nbgx,nedx,nskpx,nbgy,nedy,nskpy,
     +     ntiskp,ndamp,nsv,ifre,nxsc,nysc,nzsc,nholes,nbhx,nbhy,
     +     nbhz,idebug,dmax,vmax )
                                                  
ccc   routine to read model and source parameters
                                                                     
c     nt     # of time steps                       (integer)(returned)
c     dh     spatial discretization step           (real)   (returned)
c     dt     time discretization step              (real)   (returned)
c     ndamp  # of points to be damped              (integer)(returned)  
c     ntiskp loop increment for time samples (t)   (integer)(returned)
c     nskpx  loop increment for time samples (x)   (integer)(returned)
c     nskpy  loop increment for time samples (y)   (integer)(returned)
c     nbgx   first line in x dir with receiver     (integer)(returned)
c     nedx   last  line in x dir with receiver     (integer)(returned)
c     nbgy   first line in y dir with receiver     (integer)(returned)
c     nedy   last  line in y dir with receiver     (integer)(returned)
c     nxsc   x node point for source               (integer)(returned)
c     nysc   y node point for source               (integer)(returned)
c     nzsc   z node point for source               (integer)(returned)
c     nsv    save model every nsv timestep         (integer)(returned)
c     ifre   flag; =0 flat fs, <>0 topo            (integer)(returned)
c     nholes # of boreholes to write seismograms   (integer)(returned)
c     nbhx   array containing x-nodes of boreholes (integer)(returned)
c     nbhy   array containing y-nodes of boreholes (integer)(returned)
                                                                    
      include 'parstat'
      dimension nbhx(1),nbhy(1),nbhz(1)
      character*80 aa
c
      print *,'read from unit 22 file input.dat'
c
      read(22,*) nxt,nyt,nzt                 
      read(22,*) nt
      print *,'nt =',nt
      read(22,*) dh
      read(22,*) dt
      read(22,*) nxsc
      read(22,*) nysc
      read(22,*) nzsc
c 
c  read friction law parameters dmax and vmax
c
      read(22,*) dmax
      read(22,*) vmax
c      read(22,*) domp
      read(22,*) nbgx
      read(22,*) nedx
      read(22,*) nskpx
      read(22,*) nbgy
      read(22,*) nedy
      read(22,*) nskpy
      read(22,*) ntiskp
      read(22,*) ndamp
      read(22,*) ifre
      read(22,*) nsv
      read(22,*) nholes
      if(nholes.gt.0) then
         do 10 j=1,nholes
            read(22,*) nbhx(j), nbhy(j),nbhz(j)
 10      continue
      endif

      slip=1.
      idebug=0

      return                                                       
      end                                                         
c
c======================================================================
c             OPEN FILES
c======================================================================
c 
      subroutine open3d(io)

c     routine to open and close files

c     io   unit # (integer) (sent)

      if (io.eq.1) then

         open(21,file='verif.res')
         open(22,file='input.dat')
c     open(24,file='CHKJOB')
         open(23,file='result/ssx3d.res')
         open(26,file='result/ssy3d.res')
         open(27,file='result/ssz3d.res')
c     
         open(28,file='result/sismox.dat')
         open(29,file='result/sismoy.dat')
         open(30,file='result/sismoz.dat')
         open(31,file='result/sismox_v.dat')
         open(32,file='result/sismoy_v.dat')
         open(33,file='result/sismoz_v.dat')
         open(35,file='result/slip.res')
         open(36,file='result/sliprate.res')
         open(56,file='result/sliprate_xz.res')
         open(66,file='result/sliprate_yz.res')
c
c section temps
c
         open(37,file='result/Slipxsec.res')
         open(67,file='result/Slipxmmsec.res')
         open(57,file='result/Slipratexsec.res')
         open(38,file='result/Stresxsec.res')
         open(39,file='result/Slipysec.res')
         open(69,file='result/Slipymmsec.res')
         open(59,file='result/Slipyratesec.res')
         open(40,file='result/Stresysec.res')
         
      else
         
         close(21)
         close(22)
         close(23)
         close(26)
         close(27)
         close(28)
         close(29)
         close(30)
         close(31)
         close(32)
         close(33)
        
         close(35)
         close(36)
         close(37)
         close(38)
         close(39)
         close(40)
         
      endif
      
      return
      
      end
c
c   ====================================================================
c              WRITE MODEL 3d
c   ====================================================================
c
 
      subroutine wrpm3d(nxt,nyt,nzt,nt,dh,dt,nsou,               
     +     ndamp,ntiskp,nbgx,nedx,nskpx,nbgy,nedy,
     +     nskpy,vpe,vse,den,ifre,nxsc,nysc,nzsc,
     +     peak,ruptur,rupmax, strout, strasp,itype,
     +     dmax,vmax, xmin,xmax,ymin,ymax )
                                                                      
ccc   routine to write model and source parameters                   

c     nxt    # of node points in x-direction       (integer)(sent)
c     nyt    # of node points in y-direction       (integer)(sent)
c     nzt    # of node points in z-direction       (integer)(sent)
c     nt     # of time steps                       (integer)(sent)    
c     dh     spatial discretization step           (real)   (sent)    
c     dt     time discretization step              (real)   (sent)    
c     ndamp  # of points to be damped              (integer)(sent)      
c     ntiskp loop increment for time samples (t)   (integer)(sent)    
c     nskpx  loop increment for time samples (x)   (integer)(sent)    
c     nskpy  loop increment for time samples (y)   (integer)(sent)    
c     nbgx   first line in x dir with receiver     (integer)(sent)    
c     nedx   last  line in x dir with receiver     (integer)(sent)    
c     nbgy   first line in y dir with receiver     (integer)(sent)    
c     nedy   last  line in y dir with receiver     (integer)(sent)    
c     nxsc   x node point for source               (integer)(sent)    
c     nysc   y node point for source               (integer)(sent)    
c     nzsc   z node point for source               (integer)(sent)    
c     nsv    save model every nsv timestep         (integer)(sent)    
c     ifre   flag; =0 flat fs, <>0 topo            (integer)(sent)    
c     nsou   number of points in source            (integer)(sent)    
c     vpe(1)  lowest p-velocity                    (real)   (returned)
c     vpe(2)  highest p-velocity                   (real)   (returned)
c     vse(1)  lowest s-velocity                    (real)   (returned)
c     vse(2)  highest s-velocity                   (real)   (returned)
c     den(1)  lowest density                       (real)   (returned)
c     den(2)  highest density                      (real)   (returned)
                                                                    
      include 'parstat'
                                                             
      dimension vpe(*),vse(*),den(*)
                                                                 
      write(21,70) vpe(2)*dt/dh
      write(21,55) nxt                                               
      write(21,56) nyt                                              
      write(21,57) nzt                                             
      write(21,58) nt                                             
      write(21,59) dh                                            
      write(21,60) dt                                           
c      write(21,61) domp                                        
      write(21,62) ndamp                                      
      write(21,84) vpe(2)                                              
      write(21,85) vpe(1)                                             
      write(21,86) vse(2)                                            
      write(21,87) vse(1)                                           
      write(21,88) den(2)                                            
      write(21,89) den(1)                                           
      write(21,100) nxsc
      write(21,101) nysc
      write(21,102) nzsc
      if (ifre.eq.0) then
         write(21,48)                                                     
      else
         write(21,49)                                               
      endif
                                                                  
      write(21,28) ntiskp
                                                               
      write(21,72) nsou                                       
                                                             
  106 format(50i6)  
   28 format('skip of seismograms in time (loop counter)',i5)          
   35 format(4i8)                                                     
   36 format(10i4)                                                   
   48 format('using flat free surface boundary condition')
   49 format('using irregular free surface boundary condition')
   51 format('receiver lines in x dir')                              
   52 format('receiver lines in x dir')                             
   53 format('receiver lines in x dir')                            
   55 format('# of x nodes', i5)                                       
   56 format('# of y nodes',i5)                                        
   57 format('# of z nodes',i5)                                       
   58 format('# of time steps',i5)                                   
   59 format('discretization in space',f10.4)                       
   60 format('discretization in time' ,f10.4)                       
   61 format('dominant source period',f8.3)                    
   62 format('exp damping # of points',i5)                    
   65 format('source direction',9i5)                            
   70 format('stability criteria .5 > vp(max)*dt/dx = ',e15.7)    
   72 format('# of samples in source',i10)                            
   84 format('highest p-velocity encountered',f10.2)              
   85 format('lowest  p-velocity encountered',f10.2)              
   86 format('highest s-velocity encountered',f10.2)            
   87 format('lowest  s-velocity encountered',f10.2)                    
   88 format('highest density    encountered',f10.2)                       
   89 format('lowest  density    encountered',f10.2)                     
  100 format('source node position (x)',i5)                     
  101 format('source node position (y)',i5)                     
  102 format('source node position (z)',i5)                     
  103 format('plane wave incidence angle',f10.2)                     
  104 format('plane wave azimuth',f10.2)                     
c
c     print them out
c
      write(21,*)
      write(21,*) '   SOURCE DATA '
      write(21,*) ' '
      write(21,*)' center of fault: nxrsc,nysc,nzsc = ', nxsc,nysc,nzsc
      write(21,*)' grid size :  nxt,nyt,nzt = ', nxt,nyt,nzt
      write(21,*)' slip weaken. zone dmax   =', dmax
      write(21,*)' velocity weaken. zone vmax,   =', vmax
c      call flush(21)
                                                                     
      return                                                        
      end

c
c   ====================================================================
c              WRITE SEISMOGRAMS
c   ====================================================================
c
 
      subroutine wrss3d(i,ntiskp,nxt,nyt,nzt,dt,dx,nholes,nbhx,nbhy,
     +     nbhz,nbgx,nedx,nskpx,nbgy,nedy,nskpy,ifre,ndamp,
     $     nxsc,nysc,nzsc)
                                                                     
c     routine to save seismograms
                                                                    
c     i      current timestep                          (int)(sent)
c     ntiskp Step length in time for seismograms       (int)(sent)     
c     nxt    nodal points in x direction               (int)(sent)    
c     nyt    nodal points in y direction               (int)(sent)   
c     nzt    nodal points in z direction               (int)(sent)  
c     nskpx  loop increment in x dir for receivers     (int)(sent) 
c     nskpy  loop increment in y dir for receivers     (int)(sent)
c     nbgx   first line in x dir to contain receivers  (int)(sent)    
c     nedx   last  line in x dir to contain receivers  (int)(sent)   
c     nbgy   first line in y dir to contain receivers  (int)(sent)  
c     nedy   last  line in y dir to contain receivers  (int)(sent) 
c     dx     discretization in space                  (real)(sent)    
c     dt     discretization in time                   (real)(sent)   
c     ifre   flag; =0 flat fs, <>0 topo            (integer)(sent)    
c     ndamp  # of points to be damped              (integer)(sent)
                                                                    
      include 'parstat'
      common/strini/ strinix(nx,ny),striniy(nx,ny)

      dimension nbhx(1),nbhy(1),nbhz(1)
c
c  writes the x-slip velocity  field to file pressure
c
      if(mod(i,2).eq.1) then
         time=float(i)*dt
         delx=dx/1000.
	 write(37,'(3g14.4)') 
     $         (ii*delx,time,u2(ii,nysc,nzsc+1),ii=nbgx,nedx,nskpx)
         write(37,'()') 
	 write(67,'(3g14.4)') 
     $         (ii*delx,time,u2(ii,nysc,nzsc-1),ii=nbgx,nedx,nskpx)
         write(67,'()') 
	 write(57,'(3g14.4)') 
     $         (ii*delx,time,u1(ii,nysc,nzsc+1),ii=nbgx,nedx,nskpx)
         write(57,'()') 
	 write(38,'(3g14.4)') 
     $         (ii*delx,time,xz(ii,nysc,nzsc)+strinix(ii,nysc),
     $        ii=nbgx,nedx,nskpx)
         write(38,'()') 
	 write(39,'(3g14.4)') 
     $         (ii*delx,time,u2(nxsc,ii,nzsc+1),ii=nbgy,nedy,nskpy)
         write(39,'()') 
	 write(69,'(3g14.4)') 
     $         (ii*delx,time,u2(nxsc,ii,nzsc-1),ii=nbgy,nedy,nskpy)
         write(69,'()') 
	 write(59,'(3g14.4)') 
     $         (ii*delx,time,u1(nxsc,ii,nzsc+1),ii=nbgy,nedy,nskpy)
         write(59,'()') 
	 write(40,'(3g14.4)') 
     $         (ii*delx,time,xz(nxsc,ii,nzsc)+strinix(nxsc,ii),
     $        ii=nbgy,nedy,nskpy)
         write(40,'()') 
c
c  write isolated seismograms
c
         if(nholes.gt.0) then
            write(28,'(12g14.5)')time,(u2(nbhx(j),nbhy(j),nbhz(j)),
     $           j=1,nholes)
            write(29,'(12g14.5)')time,(v2(nbhx(j),nbhy(j),nbhz(j)),
     $           j=1,nholes)
            write(30,'(12g14.5)')time,(w2(nbhx(j),nbhy(j),nbhz(j)),
     $           j=1,nholes)
            write(31,'(12g14.5)')time,(u1(nbhx(j),nbhy(j),nbhz(j)),
     $           j=1,nholes)
            write(32,'(12g14.5)')time,(v1(nbhx(j),nbhy(j),nbhz(j)),
     $           j=1,nholes)
            write(33,'(12g14.5)')time,(w1(nbhx(j),nbhy(j),nbhz(j)),
     $           j=1,nholes)
         endif
      endif
      itt=ntiskp*(i/ntiskp)                                        
      if(itt.ne.i) return                                         
c
c write cross sections
c    
c 35 slip.res
c                         
      do j=nbgy,nedy,nskpy
         write(35,'(1g12.4)') (u2(ii,j,nzsc+1),
     +        ii=nbgx,nedx,nskpx)
c         write(35,'()') 
      enddo
c      write(35,'("==")') 
c    
c 36 sliprate.res
c                         
      do j=nbgy,nedy,nskpy
         write(36,'(1g12.4)') (u1(ii,j,nzsc+1),
     +        ii=nbgx,nedx,nskpx)
c         write(36,'()') 
      enddo
c      write(36,'("==")') 
c    
c 36 ssx3d.res
c                         
      do j=nbgy,nedy,nskpy
         write(23,'(1g12.4)') (xz(ii,j,nzsc)+strinix(ii,j),
     +        ii=nbgx,nedx,nskpx)
c         write(23,'()') 
      enddo
c      write(23,'("==")') 
c    
c 36 ssy3d.res
c                         
      do j=nbgy,nedy,nskpy
         write(26,'(1g12.4)') (yz(ii,j,nzsc),
     +        ii=nbgx,nedx,nskpx)
c         write(26,'()') 
      enddo
c      write(26,'("==")') 
c    
c 36 sliprate_yz.res
c                         
      do j=nbgy,nedy,nskpy
         write(56,'(1g12.4)') (u1(nxsc,j,ii),
     +        ii=1,nzt)
c         write(56,'()') 
      enddo
c      write(56,'("==")') 
c    
c 36 sliprate_yz.res
c                         
      do j=nbgx,nedx,nskpx
         write(66,'(1g12.4)') (u1(j,nysc,ii),
     +        ii=1,nzt)
c         write(66,'()') 
      enddo
c      write(66,'("==")') 

      return                                                   
      end                                                      
